"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultiViewerBorderColorCommand = void 0;
const CommandBase_1 = require("../CommandBase");
const state_1 = require("../../state");
class MultiViewerBorderColorCommand extends CommandBase_1.SymmetricalCommand {
    constructor(multiviewerId, color) {
        super(color);
        this.multiViewerId = multiviewerId;
    }
    serialize() {
        const buffer = Buffer.alloc(12);
        buffer.writeUInt8(this.multiViewerId, 0);
        buffer.writeUInt16BE(this.properties.red, 2);
        buffer.writeUInt16BE(this.properties.green, 4);
        buffer.writeUInt16BE(this.properties.blue, 6);
        buffer.writeUInt16BE(this.properties.alpha, 8);
        return buffer;
    }
    static deserialize(rawCommand) {
        const multiViewerId = rawCommand.readUInt8(0);
        const color = {
            red: rawCommand.readUInt16BE(2),
            green: rawCommand.readUInt16BE(4),
            blue: rawCommand.readUInt16BE(6),
            alpha: rawCommand.readUInt16BE(8),
        };
        return new MultiViewerBorderColorCommand(multiViewerId, color);
    }
    applyToState(state) {
        if (!state.info.multiviewer || this.multiViewerId >= state.info.multiviewer.count) {
            throw new state_1.InvalidIdError('MultiViewer', this.multiViewerId);
        }
        const multiviewer = state_1.AtemStateUtil.getMultiViewer(state, this.multiViewerId);
        multiviewer.borderColor = { ...this.properties };
        return `settings.multiViewers.${this.multiViewerId}.borderColor`;
    }
}
exports.MultiViewerBorderColorCommand = MultiViewerBorderColorCommand;
MultiViewerBorderColorCommand.rawName = 'MvBC';
//# sourceMappingURL=MultiViewerBorderColorCommand.js.map