"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultiViewerWindowOverlayPropertiesUpdateCommand = exports.MultiViewerWindowOverlayPropertiesCommand = void 0;
const CommandBase_1 = require("../CommandBase");
const state_1 = require("../../state");
class MultiViewerWindowOverlayPropertiesCommand extends CommandBase_1.WritableCommand {
    constructor(multiviewerId, windowIndex) {
        super();
        this.multiViewerId = multiviewerId;
        this.windowIndex = windowIndex;
    }
    serialize() {
        const buffer = Buffer.alloc(8);
        buffer.writeUInt8(this.multiViewerId, 0);
        buffer.writeUInt8(this.windowIndex || 0, 1);
        let value = 0;
        if (this.properties.labelVisible)
            value |= 0x01;
        if (this.properties.borderVisible)
            value |= 0x02;
        buffer.writeUInt8(value, 3);
        buffer.writeUInt8(this.flag, 5);
        return buffer;
    }
}
exports.MultiViewerWindowOverlayPropertiesCommand = MultiViewerWindowOverlayPropertiesCommand;
MultiViewerWindowOverlayPropertiesCommand.MaskFlags = {
    labelVisible: 1 << 0,
    borderVisible: 1 << 1,
};
MultiViewerWindowOverlayPropertiesCommand.rawName = 'CMvO';
class MultiViewerWindowOverlayPropertiesUpdateCommand extends CommandBase_1.DeserializedCommand {
    constructor(multiviewerId, windowIndex, props) {
        super(props);
        this.multiViewerId = multiviewerId;
        this.windowIndex = windowIndex;
    }
    static deserialize(rawCommand) {
        const multiViewerId = rawCommand.readUInt8(0);
        const windowIndex = rawCommand.readUInt8(1);
        const values = rawCommand.readUInt8(3);
        const props = {
            labelVisible: (values & 0x01) > 0,
            borderVisible: (values & 0x02) > 0,
        };
        return new MultiViewerWindowOverlayPropertiesUpdateCommand(multiViewerId, windowIndex, props);
    }
    applyToState(state) {
        if (!state.info.multiviewer || this.multiViewerId >= state.info.multiviewer.count) {
            throw new state_1.InvalidIdError('MultiViewer', this.multiViewerId);
        }
        const multiviewer = state_1.AtemStateUtil.getMultiViewer(state, this.multiViewerId);
        const window = multiviewer.windows[this.windowIndex];
        if (!window) {
            throw new state_1.InvalidIdError('MultiViewer Window', this.multiViewerId, this.windowIndex);
        }
        window.overlayProperties = this.properties;
        return `settings.multiViewers.${this.multiViewerId}.windows.${this.windowIndex}.overlayProperties`;
    }
}
exports.MultiViewerWindowOverlayPropertiesUpdateCommand = MultiViewerWindowOverlayPropertiesUpdateCommand;
MultiViewerWindowOverlayPropertiesUpdateCommand.rawName = 'MvOv';
//# sourceMappingURL=MultiViewerWindowOverlayPropertiesCommand.js.map