"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultiViewerWindowOverlaySafeAreaPatternCommand = void 0;
const CommandBase_1 = require("../CommandBase");
const state_1 = require("../../state");
const atemUtil_1 = require("../../lib/atemUtil");
class MultiViewerWindowOverlaySafeAreaPatternCommand extends CommandBase_1.SymmetricalCommand {
    constructor(multiviewerId, windowIndex, safeTitlePattern) {
        super({ safeTitlePattern });
        this.multiViewerId = multiviewerId;
        this.windowIndex = windowIndex;
    }
    serialize() {
        const buffer = Buffer.alloc(4);
        buffer.writeUInt8(this.multiViewerId, 0);
        buffer.writeUInt8(this.windowIndex, 1);
        buffer.writeUInt8((0, atemUtil_1.combineComponents)(this.properties.safeTitlePattern), 2);
        return buffer;
    }
    static deserialize(rawCommand) {
        const multiViewerId = rawCommand.readUInt8(0);
        const windowIndex = rawCommand.readUInt8(1);
        const safeTitlePattern = (0, atemUtil_1.getComponents)(rawCommand.readUInt8(2));
        return new MultiViewerWindowOverlaySafeAreaPatternCommand(multiViewerId, windowIndex, safeTitlePattern);
    }
    applyToState(state) {
        if (!state.info.multiviewer || this.multiViewerId >= state.info.multiviewer.count) {
            throw new state_1.InvalidIdError('MultiViewer', this.multiViewerId);
        }
        const multiviewer = state_1.AtemStateUtil.getMultiViewer(state, this.multiViewerId);
        const window = multiviewer.windows[this.windowIndex];
        if (!window) {
            throw new state_1.InvalidIdError('MultiViewer Window', this.multiViewerId, this.windowIndex);
        }
        window.safeTitlePattern = this.properties.safeTitlePattern;
        return `settings.multiViewers.${this.multiViewerId}.windows.${this.windowIndex}.safeTitlePattern`;
    }
}
exports.MultiViewerWindowOverlaySafeAreaPatternCommand = MultiViewerWindowOverlaySafeAreaPatternCommand;
MultiViewerWindowOverlaySafeAreaPatternCommand.rawName = 'StMw';
//# sourceMappingURL=MultiViewerWindowSafeAreaTypeCommand.js.map