"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.forkWebWorker = exports.WebWorkerProcess = void 0;
const _base_1 = require("./_base");
/** Functions for emulating child-process in web-workers */
class WebWorkerProcess extends _base_1.WorkerPlatformBase {
    constructor(pathToWorker) {
        super();
        try {
            // @ts-ignore
            this.worker = new window.Worker(pathToWorker);
            this.worker.onmessage = (message) => {
                if (message.type === 'message') {
                    this.emit('message', message.data);
                }
                else
                    console.log('unknown message type', message);
            };
            this.worker.onmessageerror = (error) => {
                this.emit('error', error);
            };
            this.worker.onerror = (error) => {
                this.emit('error', error);
            };
        }
        catch (error) {
            let str = (error.stack || error).toString() + '';
            if (str.match(/cannot be accessed from origin/) &&
                str.match(/file:\/\//)) {
                throw Error('Unable to create Web-Worker. Not allowed to run from local file system.\n' + str);
            }
            else {
                throw error;
            }
        }
        // this.worker.postMessage([first.value,second.value]); // Sending message as an array to the worker
    }
    kill() {
        this.worker.terminate();
        this.emit('close');
    }
    send(message) {
        this.worker.postMessage(message);
    }
}
exports.WebWorkerProcess = WebWorkerProcess;
function forkWebWorker(pathToWorker) {
    return new WebWorkerProcess(pathToWorker);
}
exports.forkWebWorker = forkWebWorker;
//# sourceMappingURL=webWorkers.js.map